!***********************************************************************
!*   
!*   Licensed Materials - Property of IBM.
!*   IBM XL Fortran Advanced Edition V10.1 for Linux (5724-M17)
!*   (C) Copyright IBM Corp. 2003, 2006.
!*   All Rights Reserved.
!*   US Government Users Restricted Rights - 
!*   Use, duplication or disclosure restricted by 
!*   GSA ADP Schedule Contract with IBM Corp.
!*
!***********************************************************************

#if !defined(__linux__) && !defined(__APPLE__)
#ifndef _AIX
#define _AIX
#endif
#endif

#ifdef _AIX

#ifndef __CHAR
#define  __CHAR     byte
#endif
#ifndef __C_PTR
#define  __C_PTR    integer(4)
#endif
#ifndef __INT32
#define  __INT32    integer(4)
#endif
#ifndef __UINT32
#define  __UINT32   integer(4)
#endif
#ifndef __DOUBLE
#define  __DOUBLE   real(8)
#endif

      integer(4), parameter :: NGPRS = 32_4
      integer(4), parameter :: NFPRS = 32_4

      !--------------------------------------------
      ! Structure to describe address space mapping
      !--------------------------------------------
      type adspace_t
        sequence
        __UINT32 :: alloc           ! Allocation flags
        __UINT32 :: srval(16)       ! Contents of all segment registers
      end type adspace_t

      !----------------------------------------------
      ! Structure to describe machine state save area
      !----------------------------------------------
      type mstsave
        sequence
        __C_PTR :: prev             ! Pointer to previous save area
        __C_PTR :: kjmpbuf          ! Pointer to saved context
        __C_PTR :: stackfix         ! Pointer to stack fix
        __CHAR :: intpri            ! Interrupt priority
        __CHAR :: backt             ! Back-track flag
        __CHAR :: rsvd(2)           ! Reserved
        __INT32 :: curid            ! Copy of "curid"
        __INT32 :: excp_type        ! Exception type (for debugger)
        __UINT32 :: iar             ! Instruction address register
        __UINT32 :: msr             ! Machine state register
        __UINT32 :: cr              ! Condition register
        __UINT32 :: lr              ! Link register
        __UINT32 :: ctr             ! Count register
        __UINT32 :: xer             ! Fixed point exception
        __UINT32 :: mq              ! Multiply/quotient register
        __UINT32 :: tid             ! TID register
        __UINT32 :: fpscr           ! Floating-point status register
        __CHAR :: fpeu              ! Floating-point "ever used"
        __CHAR :: fpinfo            ! Floating-point status flags
        __CHAR :: pad(2)            ! Reserved
        __UINT32 :: except(5)       ! Exception structure
        __C_PTR :: bus              ! Pointer to I/O bus limit register (CSR)
        __UINT32 :: o_iar           ! Old IAR  (for "longjmp" exception)
        __UINT32 :: o_toc           ! Old TOC  (for "longjmp" exception)
        __UINT32 :: o_arg1          ! Old ARG1 (for "longjmp" exception)
        __UINT32 :: excbranch       ! If not NULL, address to branch to on
                                    ! exception. Used by assembler routines
                                    ! for low cost exception handling
        __UINT32 :: fpscrx          ! Software extension to "fpscr"
        __UINT32 :: o_vaddr         ! Saved "vaddr" for "vmexception"
        __UINT32 :: u_stackp        !
        __UINT32 :: u_msr           !
        __C_PTR :: disp             ! Pointer to anchor for list of new
                                    ! "busprt" structures
        __UINT32 :: cachealign(4)   ! Unused (padding)
        type(adspace_t) :: as       ! Segment registers
        __UINT32 :: gpr(0:NGPRS-1)  ! General purpose registers
        __DOUBLE :: fpr(0:NFPRS-1)  ! Floating point registers
      end type mstsave

      !----------------------------------
      ! Structure to describe signal mask
      !----------------------------------
      type sigset_t
        sequence
        __UINT32 losigs
        __UINT32 hisigs
      end type sigset_t

      !----------------------------------------------------------------
      ! Structure to describe context for jumps within the save process
      !----------------------------------------------------------------
      type jmpbuf
        sequence
        type(mstsave) :: jmp_context
      end type jmpbuf

      !-------------------------------------------------------------
      ! Structure to describe information needed for signal handling
      !-------------------------------------------------------------
      type sigcontext
        sequence
        __INT32 :: sc_onstack       ! "sigstack" state to restore
        type(sigset_t) :: sc_mask   ! Signal mask to restore
        __INT32 :: u_error          ! "u_error" to restore
        type(jmpbuf) :: sc_jmpbuf   ! Process context to restore
      end type sigcontext

#endif /* _AIX */

#ifdef __linux__

#ifndef __ULONG64
#define __ULONG64 integer*8
#endif
#ifndef __ULONG32
#define __ULONG32 integer*4
#endif
#ifndef __ADDR64
#define __ADDR64  integer*8
#endif
#ifndef __ADDR32
#define __ADDR32  integer*4
#endif
#ifndef __INT32
#define __INT32   integer*4
#endif
#ifndef __ULONG
#ifdef __powerpc64__
#define __ULONG   integer*8
#else
#define __ULONG   integer*4
#endif
#endif
#ifndef __ADDR
#ifdef __powerpc64__
#define __ADDR   integer*8
#else
#define __ADDR   integer*4
#endif
#endif


#ifdef __powerpc64__
      integer(4), parameter :: ELF_NGREG  = 48 ! includes nip, msr, lr, etc.
      integer(4), parameter :: ELF_NFPREG = 33 ! includes fpscr
      integer(4), parameter :: _SIGSET_NWORDS = 16
      integer(4), parameter :: __SI_PAD_SIZE = 28
#else
      integer(4), parameter :: _SIGSET_NWORDS = 32
      integer(4), parameter :: __SI_PAD_SIZE = 29
#endif


      !----------------------------------------------------------------
      ! Constants that can be used in conjunction with the regs pointer
      ! in sigcontext.  Adding one of these constants to the value of
      ! the regs pointer will give the address of the item.
      ! Note:  Avoid using these constants in the gpr array of pt_regs
      ! since that would cause an array-out-of-bounds error if used in
      ! a program compiled with -C.
      !----------------------------------------------------------------

      ! General Purpose Registers
      integer, parameter :: PT_NIP = 32
      integer, parameter :: PT_MSR = 33
      integer, parameter :: PT_CTR = 35
      integer, parameter :: PT_LNK = 36
      integer, parameter :: PT_XER = 37
      integer, parameter :: PT_CCR = 38
#ifdef __powerpc64__
      integer, parameter :: PT_SOFTE = 39

      ! Floating Point Registers
      ! Each FP reg occupies 2 slots in this space
      integer, parameter :: PT_FPR0 = 48
      integer, parameter :: PT_FPR31 = (PT_FPR0 + 31)
      integer, parameter :: PT_FPSCR = (PT_FPR0 + 32 + 1)
#else
      integer, parameter :: PT_MQ  = 39

      ! Floating Point Registers
      ! Each FP reg occupies 2 slots in this space
      integer, parameter :: PT_FPR0 = 48
      integer, parameter :: PT_FPR31 = (PT_FPR0 + 2*31)
      integer, parameter :: PT_FPSCR = (PT_FPR0 + 2*32 + 1)
#endif


      !--------------------------------------------------------------
      ! Structure to hold the volatile registers during a system call
      ! or other kernel entry
      !--------------------------------------------------------------
      type pt_regs
        sequence
        __ULONG gpr(0:PT_FPSCR)      ! all registers

!       __ULONG gpr(0:31)      ! General Purpose registers
!       __ULONG nip            ! Next instruction pointer
!       __ULONG msr            ! Machine state register
!       __ULONG orig_gpr3      ! Used for restarting system calls
!       __ULONG ctr            ! Count register
!       __ULONG link           ! Link register
!       __ULONG xer            ! Fixed point exception
!       __ULONG ccr            ! Condition register
!#ifdef __powerpc64__
!       __ULONG softe          ! Soft enabled/disabled
!#else
!       __ULONG mq             ! 601 only (not used at present)
!                              ! Used on APUS to hold IPL value.
!#endif
!       __ULONG trap           ! Reason for being here
!       __ULONG dar            ! Data address register
!       __ULONG dsisr          ! Data storage interrupt status register
!       __ULONG result         ! Result of a system call
      end type

      !--------------------------------------------------------------
      ! Structure to hold the volatile registers during a system call
      ! or other kernel entry for 32-bit programs in 64-bit mode.
      !--------------------------------------------------------------
      type pt_regs32
        sequence
        __ULONG32 gpr(0:31)    ! General Purpose registers
        __ULONG32 nip          ! Next instruction pointer
        __ULONG32 msr          ! Machine state register
        __ULONG32 orig_gpr3    ! Used for restarting system calls
        __ULONG32 ctr          ! Count register
        __ULONG32 link         ! Link register
        __ULONG32 xer          ! Fixed point exception
        __ULONG32 ccr          ! Condition register
        __ULONG32 mq           ! 601 only (not used at present)
                               ! Used on APUS to hold IPL value.
        __ULONG32 trap         ! Reason for being here
        __ULONG32 dar          ! Data address register
        __ULONG32 dsisr        ! Data storage interrupt status register
        __ULONG32 result       ! Result of a system call
      end type


      !-------------------------------------------------------------
      ! Structure to describe information needed for signal handling
      !-------------------------------------------------------------
      type sigcontext
        sequence
        __ULONG     _unused(0:3)
        __INT32     signal      ! Signal number
#ifdef __powerpc64__
        __INT32     __padsignal ! padding
#endif
        __ULONG     handler
        __ULONG     oldmask
        __ADDR      regs        ! Pointer to pt_regs structure
#ifdef __powerpc64__
        integer*8   gp_regs(0:ELF_NGREG-1)
        real*8      fp_regs(0:ELF_NFPREG-1)
#endif
      end type

      !----------------------------------
      ! Structure to describe signal mask
      !----------------------------------
      type sigset_t
        sequence
        __ULONG __val(0:_SIGSET_NWORDS-1)
      end type

      !----------------------------------------------
      ! Structure to hold information about the stack
      !----------------------------------------------
      type stack_t
        sequence
        __ULONG ss_sp
        __INT32 ss_flags
#ifdef __powerpc64__
        __INT32 __padss_flags  ! padding
#endif
        __ULONG ss_size
      end type

      !------------------
      ! Userlevel context
      !------------------
      type ucontext_t
        sequence
        __ULONG uc_flags
        __ADDR uc_link        ! pointer to ucontext
        type(stack_t) :: uc_stack
#ifdef __powerpc64__
        type(sigset_t) uc_sigmask
        type(sigset_t) __unsued(0:14)
        type(sigcontext) uc_mcontext
#else
        type(sigcontext) uc_mcontext
        type(sigset_t) uc_sigmask
#endif
      end type

      !----------------------------------------------------------------
      ! Structure to hold information about a signal such as the signal
      ! number and the reason the signal was generated.
      !----------------------------------------------------------------
      type siginfo_t
        sequence
        __INT32 si_signo;      ! Signal number.
        __INT32 si_errno;      ! If non-zero, an errno value associated
                               ! with this signal, as defined in
                               ! <errno.h>.
        __INT32 si_code;       ! Signal code.
        __INT32 _pad(0:__SI_PAD_SIZE-1);

#ifdef __powerpc64__
        __INT32 __morepadding; ! _pad in 64-bit mode is in a union with
                               ! a larger structure.  Use __morepadding
                               ! to make up the difference.
#endif
      end type

#endif /* __linux__ */

#ifdef __APPLE__

#ifndef __ULONG64
#define __ULONG64 integer*8
#endif
#ifndef __INT32
#define __INT32   integer*4
#endif
#ifndef __UINT32
#define __UINT32   integer*4
#endif
#ifndef __ULONG
#ifdef __powerpc64__
#define __ULONG   integer*8
#else
#define __ULONG   integer*4
#endif
#endif
#ifndef __ADDR
#ifdef __powerpc64__
#define __ADDR   integer*8
#else
#define __ADDR   integer*4
#endif
#endif
#ifndef __DOUBLE
#define  __DOUBLE   real*8
#endif


      !--------------------------------------------------------------
      ! Structure to user threads for use in status/mutate calls.
      !--------------------------------------------------------------
      type ppc_thread_state_t
        sequence
        __UINT32 srr0          ! Instruction address register (PC)
        __UINT32 srr1          ! Machine state register (supervisor)
        __UINT32 r0
        __UINT32 r1
        __UINT32 r2
        __UINT32 r3
        __UINT32 r4
        __UINT32 r5
        __UINT32 r6
        __UINT32 r7
        __UINT32 r8
        __UINT32 r9
        __UINT32 r10
        __UINT32 r11
        __UINT32 r12
        __UINT32 r13
        __UINT32 r14
        __UINT32 r15
        __UINT32 r16
        __UINT32 r17
        __UINT32 r18
        __UINT32 r19
        __UINT32 r20
        __UINT32 r21
        __UINT32 r22
        __UINT32 r23
        __UINT32 r24
        __UINT32 r25
        __UINT32 r26
        __UINT32 r27
        __UINT32 r28
        __UINT32 r29
        __UINT32 r30
        __UINT32 r31
        __UINT32 cr            ! Condition register
        __UINT32 xer           ! User's integer exception register
        __UINT32 lr            ! Link register
        __UINT32 ctr           ! Count register
        __UINT32 mq            ! MQ register (601 only)
        __UINT32 vrsave        ! Vector Save Register
      end type

      type ppc_thread_state64_t 
        sequence
        __ULONG64 srr0        ! Instruction address register (PC)
        __ULONG64 srr1        ! Machine state register (supervisor)
        __ULONG64 r0
        __ULONG64 r1
        __ULONG64 r2
        __ULONG64 r3
        __ULONG64 r4
        __ULONG64 r5
        __ULONG64 r6
        __ULONG64 r7
        __ULONG64 r8
        __ULONG64 r9
        __ULONG64 r10
        __ULONG64 r11
        __ULONG64 r12
        __ULONG64 r13
        __ULONG64 r14
        __ULONG64 r15
        __ULONG64 r16
        __ULONG64 r17
        __ULONG64 r18
        __ULONG64 r19
        __ULONG64 r20
        __ULONG64 r21
        __ULONG64 r22
        __ULONG64 r23
        __ULONG64 r24
        __ULONG64 r25
        __ULONG64 r26
        __ULONG64 r27
        __ULONG64 r28
        __ULONG64 r29
        __ULONG64 r30
        __ULONG64 r31
        __UINT32  cr          ! Condition register
        __ULONG64 xer         ! User's integer exception register
        __ULONG64 lr          ! Link register
        __ULONG64 ctr         ! Count register
        __UINT32  vrsave      ! Vector Save Register
      end type

      !--------------------------------------------------------------
      ! Structure to hold the float point registers during a system call
      !--------------------------------------------------------------
      type ppc_float_state_t
        sequence
        __DOUBLE fpregs(0:31)
        __UINT32 fpscr_pad      ! fpscr is 64 bits, 32 bits of rubbish
        __UINT32 fpscr          ! floating point status register
      end type

      !--------------------------------------------------------------
      ! Structure to hold the vector state during a system call
      !--------------------------------------------------------------
      type ppc_vector_state_t
        sequence
        __ULONG  save_vr(0:3,0:31)
        __ULONG  save_vscr(0:3)
        __UINT32 save_pad5(0:3)
        __UINT32 save_vrvalid     ! VRs that have been saved
        __UINT32 save_pad6(0:6)
      end type

      !--------------------------------------------------------------
      ! Structure to hold the exception state during a system call
      !--------------------------------------------------------------
      type ppc_exception_state_t
        sequence
        __ULONG  dar            ! Fault registers for coredump
        __ULONG  dsisr
        __ULONG  exception      ! number of powerpc exception taken
        __ULONG  pad0           ! align to 16 bytes
        __ULONG  pad1(0:3)      ! space in PCB "just in case"
      end type

      type ppc_exception_state64_t
        sequence
        __ULONG64 dar           ! Fault registers for coredump
        __ULONG   dsisr
        __ULONG   exception     ! number of powerpc exception taken
        __ULONG  pad1(0:3)      ! space in PCB "just in case"
      end type

      !----------------------------------------------------------------
      ! Constants that can be used in conjunction with the regs pointer
      ! Note:  Avoid using these constants in the gpr array of pt_regs
      ! since that would cause an array-out-of-bounds error if used in
      ! a program compiled with -C.
      !----------------------------------------------------------------

      integer, parameter :: PPC_THREAD_STATE      = 1
      integer, parameter :: PPC_FLOAT_STATE       = 2
      integer, parameter :: PPC_EXCEPTION_STATE   = 3
      integer, parameter :: PPC_VECTOR_STATE      = 4
      integer, parameter :: PPC_THREAD_STATE64    = 5
      integer, parameter :: PPC_EXCEPTION_STATE64 = 6
      integer, parameter :: THREAD_STATE_NONE     = 7

      !-------------------------------------------------------------
      ! Structure to describe machine specific context information
      !-------------------------------------------------------------
      type mcontext
        sequence
        type(ppc_exception_state_t) :: es
        type(ppc_thread_state_t)    :: ss
        type(ppc_float_state_t)     :: fs
        type(ppc_vector_state_t)    :: vs
      end type

      type mcontext64
        sequence
#ifdef __powerpc64__
        type(ppc_exception_state64_t) :: es
#else
        type(ppc_exception_state_t) :: es
#endif
        type(ppc_thread_state64_t)    :: ss
        type(ppc_float_state_t)       :: fs
        type(ppc_vector_state_t)      :: vs
      end type

      !----------------------------------------------
      ! Structure to hold information about the stack
      !----------------------------------------------
      type stack_t
        sequence
        __ADDR  ss_sp         ! signal stack base
        __ULONG ss_size       ! signal stack length
        __INT32 ss_flags      ! SA_DISABLE and/or SA_ONSTACK
      end type

      !------------------
      ! Userlevel context
      !------------------
      type ucontext_t
        sequence
        __INT32  uc_onstack
        __UINT32 uc_sigmask         ! signal mask used by this context
        type(stack_t) uc_stack      ! stack used by this context
        __ADDR   uc_link            ! pointer to ucontext
        __ULONG  uc_mcsize          ! size of the machine context passed in
        type (mcontext) uc_mcontext ! machine specific context
      end type

      type ucontext64_t
        sequence
        __INT32  uc_onstack
        __UINT32 uc_sigmask             ! signal mask used by this context
        type(stack_t) uc_stack          ! stack used by this context
        __ADDR   uc_link                ! pointer to ucontext
        __ULONG  uc_mcsize              ! size of the machine context passed in
        type (mcontext64) uc_mcontext64 ! machine specific context
      end type

      !----------------------------------------------------------------
      ! Structure to hold information about a signal such as the signal
      ! number and the reason the signal was generated.
      !----------------------------------------------------------------
      type siginfo_t
        sequence
        __INT32  si_signo      ! signal number.
        __INT32  si_errno      ! errno association
        __INT32  si_code       ! signal code.
        __INT32  si_pid        ! sending process
        __UINT32 si_uid        ! sender's ruid
        __INT32  si_status     ! exit value
        __ADDR   si_addr       ! faulting instruction
        __INT32  si_value;     ! signal value
        __ULONG  si_band       ! band event for SIGPOLL
        __INT32  pad(0:6)      ! RFU
      end type

#endif /* __APPLE__ */
