!***********************************************************************
!*   
!*   Licensed Materials - Property of IBM.
!*   IBM XL Fortran Advanced Edition V10.1 for Linux (5724-M17)
!*   (C) Copyright IBM Corp. 2003, 2006.
!*   All Rights Reserved.
!*   US Government Users Restricted Rights - 
!*   Use, duplication or disclosure restricted by 
!*   GSA ADP Schedule Contract with IBM Corp.
!*
!***********************************************************************

@PROCESS XFLAG(HPFLINTR)
!IBM* SOURCEFORM(FREE(F90))
!
! NOTE: The fpscr constants defined in this file are specific to the
!       IBM RS/6000 family of processors.
!       They should correspond to the ones defined in:
!       /usr/include/float.h
!       /usr/include/fptrap.h
!       /usr/include/fpxcp.h
!
!
MODULE XLF_FP_UTIL
   INTEGER(4), PARAMETER :: FPSCR_KIND = 4
   INTEGER(4), PARAMETER :: FP_MODE_KIND = 4
   !
   !
   !      Values for the IEEE Rounding Modes
   !
   !      FP_RND_RN = Round toward nearest (default) = 0x00
   !      FP_RND_RZ = Round toward zero = 0x01
   !      FP_RND_RP = Round toward plus infinity = 0x02
   !      FP_RND_RM = Round toward minus infinity = 0x03
   !
   INTEGER(FPSCR_KIND), PARAMETER :: FP_RND_RN		= Z"00000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_RND_RZ		= Z"00000001"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_RND_RP		= Z"00000002"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_RND_RM		= Z"00000003"
   !
   !      Bit mask to test the current rounding mode
   !
   INTEGER(FPSCR_KIND), PARAMETER :: FP_RND_MODE        = Z"00000003"
   !
   !      Bits to enable IEEE exceptions
   !
   INTEGER(FPSCR_KIND), PARAMETER :: TRP_INEXACT	= Z"00000008"
   INTEGER(FPSCR_KIND), PARAMETER :: TRP_DIV_BY_ZERO	= Z"00000010"
   INTEGER(FPSCR_KIND), PARAMETER :: TRP_UNDERFLOW	= Z"00000020"
   INTEGER(FPSCR_KIND), PARAMETER :: TRP_OVERFLOW	= Z"00000040"
   INTEGER(FPSCR_KIND), PARAMETER :: TRP_INVALID	= Z"00000080"
   !
   !      Bit mask to test if any exceptions are enabled
   !
   INTEGER(FPSCR_KIND), PARAMETER :: FP_ENBL_SUMM	= Z"000000F8"
   !
   !      Machine specific invalid exception detail bits
   !
   !      FP_INV_SNAN       Signalling NaN
   !      FP_INV_ISI        Infinity - Infinity
   !      FP_INV_IDI        Infinity / Infinity
   !      FP_INV_ZDZ        0/0
   !      FP_INV_IMZ        Infinity * 0
   !      FP_INV_CMP        Unordered Compare
   !      FP_INV_SQRT       Square Root of negative number
   !      FP_INV_CVI        Conversion to integer error
   !      FP_INV_VXSOFT     Software request
   !
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_SNAN	= Z"01000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_ISI		= Z"00800000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_IDI		= Z"00400000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_ZDZ		= Z"00200000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_IMZ		= Z"00100000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_CMP		= Z"00080000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_SQRT	= Z"00000200"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_CVI		= Z"00000100"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INV_VXSOFT	= Z"00000400"
   !
   !      IEEE required summary bits.
   !
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INVALID		= Z"20000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_OVERFLOW	= Z"10000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_UNDERFLOW	= Z"08000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_DIV_BY_ZERO	= Z"04000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_INEXACT		= Z"02000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_ALL_IEEE_XCP	= &
     & IOR(IOR(IOR(IOR(FP_INVALID,FP_OVERFLOW),FP_UNDERFLOW),&
     & FP_DIV_BY_ZERO),FP_INEXACT)
   INTEGER(FPSCR_KIND), PARAMETER :: FP_COMMON_IEEE_XCP	= &
     & IEOR(FP_ALL_IEEE_XCP, FP_INEXACT)
   !
   !      Machine specific summary bits
   !
   INTEGER(FPSCR_KIND), PARAMETER :: FP_ANY_XCP		= Z"80000000"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_ALL_XCP		= Z"BFF80700"
   INTEGER(FPSCR_KIND), PARAMETER :: FP_COMMON_XCP	= &
     & IEOR(FP_ALL_XCP, FP_INEXACT)

   !      fp_trap() constants
   !
   !      FP_TRAP_SYNC     	1	precise trapping on
   !      FP_TRAP_OFF      	0	trapping off
   !      FP_TRAP_QUERY    	2	query trapping mode
   !      FP_TRAP_IMP     	3	non-recoverable imprecise trapping on
   !      FP_TRAP_IMP_REC    	4	recoverable imprecise trapping on
   !      FP_TRAP_FASTMODE	128	select fastest available mode
   !      FP_TRAP_ERROR          -1	error condition
   !      FP_TRAP_UNIMPL         -2	requested mode not available
   !
   INTEGER(4), PARAMETER :: FP_TRAP_SYNC	= 1
   INTEGER(4), PARAMETER :: FP_TRAP_OFF 	= 0
   INTEGER(4), PARAMETER :: FP_TRAP_QUERY	= 2
   INTEGER(4), PARAMETER :: FP_TRAP_IMP		= 3
   INTEGER(4), PARAMETER :: FP_TRAP_IMP_REC	= 4
   INTEGER(4), PARAMETER :: FP_TRAP_FASTMODE	= 128
   INTEGER(4), PARAMETER :: FP_TRAP_ERROR	= -1
   INTEGER(4), PARAMETER :: FP_TRAP_UNIMPL	= -2

   ! The n in INTEGER(n) in this interface must match the FP_MODE_KIND.
   INTERFACE
      INTEGER(4) FUNCTION FP_TRAP(%val(TRP_MODE))
      INTEGER(4), INTENT(IN) :: TRP_MODE
      END FUNCTION FP_TRAP
   END INTERFACE

   INTRINSIC GET_FPSCR, SET_FPSCR
   INTRINSIC GET_ROUND_MODE, SET_ROUND_MODE
   INTRINSIC GET_FPSCR_FLAGS, SET_FPSCR_FLAGS, CLR_FPSCR_FLAGS

END MODULE XLF_FP_UTIL
