/**********************************************************************/
/*                                                                    */
/*   Licensed Materials - Property of IBM.                            */
/*   IBM XL Fortran Advanced Edition V10.1 for Linux (5724-M17)       */
/*   (C) Copyright IBM Corp. 2003, 2006.                              */
/*   All Rights Reserved.                                             */
/*   US Government Users Restricted Rights -                          */
/*   Use, duplication or disclosure restricted by                     */
/*   GSA ADP Schedule Contract with IBM Corp.                         */
/*                                                                    */
/**********************************************************************/

#include <pthread.h>
#include <sys/types.h>

/*****************************************************************/
/* The type definition for the layout of the Fortran threadlocal */
/* control structure                                             */
/*****************************************************************/

typedef struct {
  int key;
  int flags;
  int unused_1;
  int unused_2;
} FORT_LOCAL_COMMON;

#define KEY_EXISTS 0x1

/*****************************************************************/
/* For each threadlocal common block, declare a control          */
/* structure.                                                    */
/* Also, each block will need a type definition for it's specific*/
/* layout.                                                       */
/*****************************************************************/
extern FORT_LOCAL_COMMON block1;
typedef struct {
  int i;
  int j;
} FORT_BLOCK1;

extern FORT_LOCAL_COMMON block2;
typedef struct {
  float r;
} FORT_BLOCK2;

/*****************************************************************/
/* Start of the C main routine...                                */
/*****************************************************************/
cprint_common()
{
  FORT_BLOCK1 *ptr1;            /*A pointer to threadlocal block1*/
  FORT_BLOCK2 *ptr2;            /*A pointer to threadlocal block2*/
  uint key_exists;
  static pthread_mutex_t mutex=PTHREAD_MUTEX_INITIALIZER;
  

  /***************************************************************/
  /* Before accessing the threadlocal blocks, we need to ensure  */
  /* they have a key, and have been allocated.                   */
  /***************************************************************/
  pthread_mutex_lock(&mutex);
  key_exists = (block1.flags & KEY_EXISTS);
  pthread_mutex_unlock(&mutex);

  if (key_exists)                 
    {
     ptr1 = pthread_getspecific(block1.key);
     if (ptr1 != NULL) 
     {
        printf("Fortran common block1 \n");
        printf("i = %i\n",(*ptr1).i);
        printf("j = %i\n",(*ptr1).j);
     }
  }    
     
  pthread_mutex_lock(&mutex);
  key_exists = (block2.flags & KEY_EXISTS);
  pthread_mutex_unlock(&mutex);

  if (key_exists)                 
    {
     ptr2 = pthread_getspecific(block2.key);
     if (ptr2 != NULL) 
     {
        printf("Fortran common block2 \n");
        printf("r = %f\n",(*ptr2).r);
     }
    }
}
